###############################################################################
###############################################################################
#####                            Dialogos.tcl
###############################################################################
###############################################################################
##### The contents of this file are adapted from an example in Brent Welch's
##### book "Practical Programming in Tcl/Tk". I made the changes without
##### knowing very well what I was doing, so please, don't blame him for all
##### the weirdness.
###############################################################################
##### Copyright 1999-2008 Brent Welch - Andres Garcia.  fandom@telefonica.net
##### The contents of this file are distributed under the terms of the LGPL
###############################################################################

namespace eval Dialogos {

set ext $::getleftState(imgExt)

set imaArr [image create photo \
        -file [file join "$dirGetleft(icons)" arriba.$ext]]
set imaNew [image create photo \
        -file [file join "$dirGetleft(icons)" nuevo.$ext]]

###############################################################################
# SelectDirNative
#     Does the work by using a Windows or Mac dialog
#
# Parameters:
#     initialDir: The directory in which the dialog will open itself.
#     parent: The window over which it will appear.
#
# Returns:
#     The chosen path or an empty string if the user cancels.
#
# Side effects:
#     If you select a non-existing directory, it will be created for you.
###############################################################################
proc SelectDirNative {initialDir parent} {
    global labelTitles labelMessages indexButtons

    set chosenDir [tk_chooseDirectory -title $labelTitles(directory)            \
            -parent $parent -initialdir $initialDir]

    if {$chosenDir==""} return

    if {![file exist $chosenDir]} {
        set what [Dialogos::Dialogo $parent.question -icon question -type yesno \
                -message $labelMessages(unknown) -title $labelTitles(unknown)]
        if {$what=="no"} {
            set initialDir $chosenDir
            while {![file exists $initialDir]} {
                set initialDir [file dirname $initialDir]
            }
            return [SelectDirWindows $initialDir $parent]
        }
        file mkdir $chosenDir
    }
    return $chosenDir
}

###############################################################################
# SelectDirectory
#    Opens a dialog window which allows the user to choose one directory. If
#    needed, the directory is created.
#
# Parameter:
#    initialDir: directory where the dialog should open itself.
#    parent: the parent window of the dialog, it defaults to the main
#    window of the app.
#
# Returns
#    The full path of the chosen directory.
###############################################################################
proc SelectDirectory {{initialDir {} } {parent {.} } } {
    variable fileselect
    global tcl_patchLevel getleftState
    global dirGetleft getleftOptions labelMenus indexButtons
    global env labelButtons labelTitles labelDialogs labelMessages

    if {$getleftState(os)!="unix"} {
        return [SelectDirNative $initialDir $parent]
    }

    catch {destroy .fileselect}
    set win [toplevel .fileselect]

    set coord(x) [winfo rootx $parent]
    set coord(y) [winfo rooty $parent]

    wm title $win $labelTitles(directory)
    wm resizable $win 0 0
    wm geometry $win +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]

    set extFrame [ttk::frame $win.t]

    set topFrame           [ttk::frame $extFrame.top]
    set labelCurrent       [ttk::label $topFrame.labelCurrent                  \
            -text "$labelDialogs(current): "]
    set fileselect(dirEnt) [ttk::label $topFrame.entry -relief sunken -width 35\
            -anchor w]

    set upDir  [ttk::button $topFrame.up  -image $::Dialogos::imaArr -command {
        Dialogos::fileselectList [file dirname $Dialogos::fileselect(dir)]
        Dialogos::fileselectOK
    } -style Toolbutton]
    set newDir [ttk::button $topFrame.new -image $::Dialogos::imaNew -command {
        ::Dialogos::NewDir
    } -style Toolbutton]

    BalloonHelp::set_balloon $upDir  $labelMessages(up)
    BalloonHelp::set_balloon $newDir $labelMessages(newDir)

    set centFrame [ttk::frame $extFrame.central]
    set fileselect(dirList) [listbox $centFrame.list                         \
            -yscrollcommand [list $centFrame.scroll set]                     \
            -bg $getleftOptions(bg) -fg $getleftOptions(fg) -height 10
    ]

    set lbFont [$fileselect(dirList) cget -font]
    regsub {bold} $lbFont {} lbFont
    $fileselect(dirList) configure -font $lbFont

    set fileselect(scroll) [ttk::scrollbar $centFrame.scroll -command [
            list $fileselect(dirList) yview
    ]]

    set botFrame [ttk::frame $extFrame.bottonFrame]
    set labelDir [ttk::label $botFrame.labelDir -text "$labelDialogs(dir): "]
    set fileselect(pathEnt) [ttk::entry $botFrame.dirEntry                   \
           -width 27 -textvariable Dialogos::fileselect(path)]
    menuEntry::menuEntry $fileselect(pathEnt)

    # Set up bindings to invoke OK and Cancel
    bind $fileselect(pathEnt) <Return> {
        if {[Dialogos::fileselectOK]==1} {
            set Dialogos::fileselect(done) 1
        }
    }
    bind $fileselect(pathEnt) <Control-c> Dialogos::fileselectCancel
    focus $fileselect(pathEnt)

    set accept [ttk::button $botFrame.ok -width 10                             \
            -textvariable labelButtons(select) -underline $indexButtons(select)\
            -command {
                if {[Dialogos::fileselectOK]==1} {
                    set Dialogos::fileselect(done) 1
                }
            }
    ]

    set cancel [ttk::button $botFrame.cancel -width 10                         \
            -textvariable labelButtons(cancel) -underline $indexButtons(cancel)\
            -command {set Dialogos::fileselect(done) 0}
    ]

    wm protocol $win WM_DELETE_WINDOW "$cancel invoke"

    if {$::tcl_version>=8.5} {
        grid anchor $extFrame c
    }

    pack $extFrame -fill x -fill y -expand true -ipadx 4 -ipady 4

    grid $topFrame -sticky ew
    grid $labelCurrent  $fileselect(dirEnt) -sticky e
    grid $upDir  -row 0 -column 2 -padx 5
    grid $newDir -row 0 -column 3

    grid $centFrame -sticky ew -pady 5
    grid $fileselect(dirList) -sticky news -row 0 -column 0 -pady 2
    grid $fileselect(scroll)  -sticky nsw  -row 0 -column 1 -pady 2
    grid columnconfigure $centFrame 0 -weight 1

    grid $botFrame -sticky ew
    grid $labelDir            -sticky w  -row 0 -column 0
    grid $fileselect(pathEnt) -sticky ew -row 0 -column 1 -padx 4
    grid $accept              -sticky e  -row 0 -column 2
    grid $cancel              -sticky e  -row 1 -column 2
    grid columnconfigure $botFrame 1 -weight 1

    # A single click to listbox so the user can use arrow keys
    bind $fileselect(pathEnt) <Tab>            "focus $fileselect(dirList) ; list select set 0 ; break"
    bind $fileselect(dirList) <Return>         "Dialogos::fileselectTmp  ; break"
    bind $fileselect(dirList) <KP_Enter>       "Dialogos::fileselectTmp  ; break"
    bind $fileselect(dirList) <space>          "Dialogos::fileselectTake ; break"
    bind $fileselect(dirList) <Tab>            "focus $accept ; break"
    bind $fileselect(dirList) <Button-1>        {focus %W}
    bind $fileselect(dirList) <Double-Button-1> {Dialogos::fileselectTmp ; break }
    bind $extFrame            <KeyPress-Prior> "$fileselect(dirList) yview scroll -1 pages;break"
    bind $extFrame            <KeyPress-Next>  "$fileselect(dirList) yview scroll  1 pages;break"
    bind $accept              <Tab>            "focus $cancel ; break"
    bind $cancel              <Tab>            "focus $fileselect(pathEnt) ; break"
    bind $win                 <Escape>         "$cancel invoke"

    # Inicializar las variables

    set fileselect(path) {}
    if {($initialDir!="")&&([file exists $initialDir])} {
        set dir $initialDir
    } else {
        set dir $env(HOME)
    }

    set fileselect(dir) {}
    set fileselect(done) 0

    # Wait for the listbox to be visible so
    # we can provide feedback during the listing
    tkwait visibility $fileselect(dirList)
    fileselectList $dir

    grab $win
    tkwait variable Dialogos::fileselect(done)
    grab release $win

    destroy $win
    update     
    if {$fileselect(done)==1} {
        return $fileselect(path)
    }
    return
}

###############################################################################
# NewDir
#    Opens a dialog box to create a new directory.
###############################################################################
proc NewDir {} {
    variable fileselect
    global labelTitles labelButtons getleftOptions
    variable done

    set coord(x) [winfo rootx .fileselect]
    set coord(y) [winfo rooty .fileselect]

    set dialog  [toplevel .dialog]
    wm title $dialog $labelTitles(newDir)
    wm resizable $dialog 0 0
    wm geometry  $dialog +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]

    set done 0

    set marcoEx [ttk::frame $dialog.marcoEx]
    set marco   [ttk::frame $marcoEx.marco -relief groove]
    set marcoIn [ttk::frame $marco.marcoIn]
    set dirEnt  [ttk::entry $marcoIn.dirent -width 25]

    set botones  [ttk::frame  $dialog.botones]
    set aceptar  [ttk::button $botones.aceptar  -textvariable labelButtons(ok)     \
            -width 8 -command {set Dialogos::done 1}]
    set cancelar [ttk::button $botones.cancelar -textvariable labelButtons(cancel) \
            -width 8 -command {set Dialogos::done 0}]

    pack $cancelar -side right -padx 9 -pady 5
    pack $aceptar  -side right -pady 5
    pack $botones  -side bottom -fill x

    pack $dirEnt
    pack $marcoIn -padx 10 -pady 10
    pack $marco -side bottom
    pack $marcoEx -ipadx 10 -ipady 5

    bind $dialog <Escape> "$cancelar invoke"
    bind $dialog <Return> "$aceptar  invoke"

    focus $dirEnt
    grab $dialog
    tkwait variable Dialogos::done

    if {$done==1} {
        set dir [$dirEnt get]
        if {$dir!=""} {
            if {[catch {file mkdir [file join $fileselect(dir) $dir]} error]} {
                Dialogos::Dialogo .error -title $labelTitles(error) -icon error \
                        -message $error
            } else {
                Dialogos::fileselectList $fileselect(dir)
            }
        }
    }
    grab release $dialog
    destroy $dialog
    return
}

###############################################################################
# fileselectList
#    Puts into the dialog box the directories found in the current one.
#
# Parameter
#    dir: Current directory.
##############################################################################
proc fileselectList {dir} {
    variable fileselect
    global   labelTitles labelMessages

    # Update directory
    $fileselect(dirEnt) configure -text [file nativename $dir]

    # Give the user some feedback
    set fileselect(dir) $dir
    $fileselect(dirList) delete 0 end
    $fileselect(dirList) insert 0 Searching...
    update idletasks

    $fileselect(dirList) delete 0

    # Add father directory and scan the current one
    if {!([regexp {^((.:)?(/))$} $fileselect(dir)])} {
        $fileselect(dirList) insert end ..
    } else {
        $fileselect(dirList) insert end /
    }
    if {[catch {glob -nocomplain $fileselect(dir)/*/} ficheros]} {
        Dialogos::Dialogo .fileselect.error -type ok -icon error        \
                -message $labelMessages(noReadDir)                      \
                -title $labelTitles(error)
        fileselectList [file dirname $dir]
        return
    }

   # Show results
   foreach f [lsort -dictionary $ficheros] {
       $fileselect(dirList) insert end [file tail $f]
   }
   return
}

###############################################################################
# fileselectOk
#    This procedure is invoked when a directory is selected, if needed it asks
#    whether the user wants to create it.
##############################################################################
proc fileselectOK { } {
    variable fileselect
    global labelTitles labelMessages

    if {$fileselect(path)==""} {
        fileselectTake
        if {$fileselect(path)==""} return 
        if {[fileselectOK]==1} {
	      set fileselect(done) 1
        }
        return
    }

    # El directorio padre tiene tratamiento especial
    if {[regexp {\.\./?} $fileselect(path)]} {
        set fileselect(path) {}
        fileselectList [file dirname $fileselect(dir)]
        return
    }

    set path [file join $fileselect(dir) $fileselect(path)]

    if {![file exists $path]} {
        set decision [Dialogos::Dialogo .fileselect.question -icon question    \
                -message $labelMessages(unknown) -title $labelTitles(unknown)  \
                -type yesno]
        switch $decision {
            yes {
                file mkdir $path
                set fileselect(path) $path
                set fileselect(done) 1
                return
            }
            no return
        }
    }
    if {![file writable $path]} {
        Dialogos::Dialogo .fileselect.error -type ok -icon error            \
                -message $labelMessages(noWrite) -title $labelTitles(error)
        set fileselect(path) ""
        return
    }
    if {[file isdirectory $path]} {
        set fileselect(done) 1
        set fileselect(path) $path
    }
    return
}

###############################################################################
# fileselectCancel
#    Procedure to cancel the selection
##############################################################################
proc fileselectCancel {} {
    variable fileselect

    set fileselect(done) 1
    set fileselect(path) {}

    return
}

###############################################################################
# fileselectClick
#    Select the clicked item in the directory list.
#
# Parameter
#    y: point where the user clicked.
##############################################################################
proc fileselectClick { y } {
    variable fileselect

    set fileselect(path) [$fileselect(dirList) get [$fileselect(dirList) nearest $Y]]
    focus $fileselect(pathEnt)

    return
}

###############################################################################
# fileselectTake
#    Takes the selected item from the directory list and puts it in the
#    path entry.
##############################################################################
proc fileselectTake {} {
    variable fileselect

    set seleccion [$fileselect(dirList) curselection]
    if {$seleccion!=""} {
        set fileselect(path) [$fileselect(dirList) get $seleccion]
    }
    focus $fileselect(pathEnt)

    return
}

###############################################################################
# fileselectTmp
#    Invoked when the user doubleclicks on an item in the directory list,
#    takes the directory and makes it the current one.
##############################################################################
proc fileselectTmp {} {
    variable fileselect

    fileselectTake
    if {[string match \.\. $fileselect(path)] } {
        fileselectList [file dirname $fileselect(dir)]
    } else {
        fileselectList [file join $fileselect(dir) $fileselect(path)]
    }
    set fileselect(path) ""

    return
}

################################################################################
# DialogCallback
#     The tile dialog require a callback to process the pressed button, this is
#     it
#
# Parameter
#     button: The pressed button.
#
# Side effect
#     Puts in Dialogos::pressedButton the right value
################################################################################
proc DialogCallback {button} {
    variable pressedButton

    set pressedButton $button

    return
}

################################################################################
# Dialogo
#     Invokes the tile dialog button in Unix, and Tk's one in Mac and Win.
#     It tries to be an "almost" drop-in replacement for tk_messageBox.
#
# Parameters
#     args: The pameters to pass to the real dialog command.
#
# Returns:
#     The pressed button.
################################################################################
proc Dialogo {args} {
    variable pressedButton
    global   getleftState

    # Used to be Tile had a dialog command
    if {($getleftState(os)!="unix")||([info commands ::ttk::dialog] eq "")} {
        set parent [lrange $args 0 0]
        regexp {(.+)(\.)} $parent nada parent
        if {![winfo exists $parent]} {
            set parent .
        }
        regsub [lrange $args 0 0] {.*?$} parent {}
        return [eval tk_messageBox [lrange $args 1 end] -parent $parent]
    }

    set path [lindex $args 0]
    regsub {\.\.} $path {.} path

    eval ttk::dialog $path [lrange $args 1 end]                  \
            -command Dialogos::DialogCallback
    tkwait variable Dialogos::pressedButton

    return $pressedButton
}

}
