###############################################################################
###############################################################################
##                          EnterUrl
###############################################################################
###############################################################################
## Takes care of the dialog that prompts the user for an Url to download.
###############################################################################
###############################################################################
## (c) 2002-2008 Andrs Garca Garca. fandom@telefonica.net
## You may distribute the contents of this file under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval EnterUrl {

###############################################################################
# EnterUrlChangeDir {currentDir}
#    Invoked when the user want to change the directory in which the site
#    will be saved.
#
# Side effect:
#    If the user actually changes the directory, the new one will be shown
#    in the 'Enter Url' dialog
###############################################################################
proc EnterUrlChangeDir {} {
    global   env
    variable window

    if {$window(dir)!=$env(HOME)} {
        set currentDir [file dirname $window(dir)]
    } else {
        set currentDir $env(HOME)
    }

    set window(baseDir) [Dialogos::SelectDirectory $currentDir .enterUrlWindow]

    if {$window(baseDir)!=""} {
        set window(dir)   [file nativename $window(baseDir)]
    }

    return
}

###############################################################################
# EnterUrlWindow
#    Creates a dialog box where you can write or paste the desired url.
#
# Parameter:
#    parent: The path of the toplevel window over which it will appear.
#    url:    The url that will appear in the dialog, if none is given it will
#            look in the selection.
#
# Returns:
#    The path of the entry in the combobox.
################################################################################
proc EnterUrlWindow {parent url} {
    global labelButtons labelTitles labelFrames labelDialogs labelMenus
    global urlHistory getleftState getleftOptions dirGetleft
    global indexButtons indexDialogs
    variable window

    set coord(x) [winfo rootx $parent]
    set coord(y) [winfo rooty $parent]

    set win [toplevel .enterUrlWindow]
    keynav::enableMnemonics $win

    wm title $win $labelTitles(enterUrl)
    wm resizable $win 0 0
    wm geometry  $win +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]

    set marcoEx  [ttk::frame $win.marcoEx -class ExternalFrame]

    set urlFrame [ttk::labelframe $marcoEx.marco -labelanchor nw                          \
            -labelwidget [ttk::label $marcoEx.marcolb -textvariable labelFrames(webPage)]]
    raise $marcoEx.marcolb
    set urlCombo [ttk::combobox $urlFrame.url -values $urlHistory  -width 50              \
            -textvariable EnterUrl::window(url) -exportselection 1]
    $urlCombo set $url
    menuEntry::menuEntry $urlCombo
    set startCheck [ttk::checkbutton $urlFrame.start -variable getleftState(startNow)     \
            -textvariable labelDialogs(startNow) -underline $indexDialogs(startNow)]

    set dirFrame    [ttk::labelframe $marcoEx.dirFrame -labelanchor nw                    \
            -labelwidget [ttk::label $marcoEx.dirFramelb -textvariable labelFrames(dir)]]
    raise $marcoEx.dirFramelb
    set dirEntry    [ttk::entry $dirFrame.entry -width 47                                 \
            -textvariable EnterUrl::window(dir) -exportselection 1]
    menuEntry::menuEntry $dirEntry

    set dirButton [ttk::button $dirFrame.button -command "EnterUrl::EnterUrlChangeDir"    \
            -image $iconNS::iconImages(folderm) -style Toolbutton
    ]
    set window(dir)   [file nativename $dirGetleft(lastDir)]

    ::BalloonHelp::set_balloon $dirButton $labelButtons(select)

    $dirEntry xview moveto 1.0

    set buttons [ttk::frame  $marcoEx.buttons -class BottonFrame]
    set butLef  [ttk::frame  $buttons.left]
    set butRig  [ttk::frame  $buttons.right]
    set start   [ttk::button $butRig.start   -textvariable labelButtons(start)            \
            -width 8 -command "EnterUrl::EnterUrlControl 1"                               \
            -underline $indexButtons(start)]
    set cancel  [ttk::button $butRig.cancel  -textvariable labelButtons(cancel)           \
            -width 8 -command "EnterUrl::EnterUrlControl 0"                               \
            -underline $indexButtons(cancel)]
    set options [ttk::button $butLef.options -image $iconNS::iconImages(confm)            \
            -command "Opciones::ChooseOptions ::downOptionsTemp $win" -style Toolbutton]
    ::BalloonHelp::set_balloon $options $labelButtons(options)

    set window(win)        $win
    set window(combo)      $urlCombo
    set window(dirEntry)   $dirEntry
    set window(startCheck) $startCheck

    bind $urlCombo <Return>   "focus $dirEntry ; break"
    bind $urlCombo <KP_Enter> "focus $dirEntry"
    bind $urlCombo <FocusIn>  "$urlCombo selection range 0 end"


    bind $dirEntry       <Return>   "focus $start"
    bind $dirEntry       <KP_Enter> "focus $start"

    bind $win            <Escape>   "$cancel invoke"

    if {$::tcl_version>=8.5} {
        grid anchor $marcoEx  c
        grid anchor $urlFrame c
        grid anchor $dirFrame c
    }

    grid $marcoEx
    grid $urlFrame    -padx 10     -sticky news
    grid $urlCombo    -pady {8  5} -padx 20
    grid $startCheck               -padx 18 -sticky w

    grid $dirFrame  -padx 10 -sticky news
    grid $dirEntry                               
    grid $dirButton -row 0 -column 1 -padx {8 0}

    grid $buttons       -sticky ew -padx 7
    grid $butLef        -sticky w
    grid $butRig        -sticky e                  -row 0 -column 1
    grid $options                  -padx 2
    if {$getleftState(os) ne "mac"} {
        grid $start  $cancel -sticky e  -padx 2 -pady 3
    } else {
        grid $cancel $start  -sticky e  -padx 2 -pady 3
    }

    grid columnconfigure $buttons 1 -weight 1

    focus $urlCombo

    return $urlCombo
}

###############################################################################
# EnterUrlControl
#    Checks the url entered and complains if it is not valid.
#    In practice, in considers just about anything to be valid.
#
# Parameter
#    done: '1' if the user clicked on start, '0' if he cancelled.
################################################################################
proc EnterUrlControl {done} {
    global labelTitles labelMessages
    global dirGetleft  directories getleftState
    variable window

   if {($done==0)||($window(url)=="")} {
        destroy $window(win)
        return
    }

    set requestedUrl [string trim $window(url)]
    set parsedUrl    [HtmlParser::ParseUrl $requestedUrl]

    if {$parsedUrl==1} {
        Dialogos::Dialogo $window(win).error -title $labelTitles(error) -type ok\
                -message "$labelMessages(noWWW)\n$requestedUrl" -icon error
        return
    }

    set dir $window(dir)
    if {$dir==""} {
        Dialogos::Dialogo $window(win).error -type ok -icon error               \
                -title $labelTitles(error) -message "Please, enter a directory"
        return
    }
    regsub -all {\\} $dir             {/} window(baseDir)
    regsub      {/$} $window(baseDir) {}  window(baseDir)
    if {![file exists $window(baseDir)]} {
       set what [Dialogos::Dialogo $window(win).question -icon question       \
            -message $labelMessages(unknown) -title $labelTitles(unknown)     \
            -type yesno]
       switch $what {
            yes {
                file mkdir $window(baseDir)
            }
            no return
	  }
    } else {
       if {![file isdirectory $window(baseDir)]} {
           Dialogos::Dialogo $window(win).error -type ok -icon error          \
                    -title $labelTitles(error)                                \
                    -message "Please, enter a directory"
           return
       }
    }
    destroy $window(win)

    regsub -all {\n}   $requestedUrl {}  requestedUrl
    regsub -all {\\}   $requestedUrl {/} requestedUrl
    regsub -all {\x00} $requestedUrl {}  requestedUrl

    set domain [string tolower [lindex [HtmlParser::ParseUrl $requestedUrl] 1]]

    PutUrlInHistory $requestedUrl

    PutUrlInQueue $requestedUrl $window(baseDir) [array get ::downOptionsTemp]

    set dirGetleft(lastDir) $window(baseDir)

    if {($getleftState(downloading)==0)&&($getleftState(startNow)==1)} {
        StartDownload $requestedUrl
    }

    return
}

###############################################################################
# EnterUrl
#    Creates a dialog box where you can write or paste the desired url.
#
# Parameters
#    map: '1' if the user wants a map of the site.
#    parent: The window over which the dialog should appear.
#    url: The url to download if available.
#    label: The tag of the url in the 'Choose Files' dialog.
#
# Returns:
#    The url entered by the user.
###############################################################################
proc EnterUrl {{map 0} {parent .} {url ""} {label ""}} {
    global getleftOptions downOptions downOptionsTemp getleftState urlHistory
    variable window

    catch "unset ::downOptionsTemp"
    array set downOptionsTemp [array get downOptions]
    set downOptionsTemp(map) $map

    if {[winfo exists .enterUrlWindow]} {
        $window(combo) configure -values $urlHistory
        wm deiconify $window(win)
    } else {
        if {$url==""} {
            set url [menuEntry::ReadSel]
        }
        EnterUrlWindow $parent $url
    }
    if {$getleftState(downloading)==1} {
        $window(startCheck) configure -state disabled
    } else {
        $window(startCheck) configure -state normal
    }

    return
}
}
