###############################################################################
###############################################################################
##                              getWizard.tcl
###############################################################################
###############################################################################
## Includes procedures to implement the configuration wizard, it was created
## using tkwizard and tkwizthing by Bryan Oakley oakley_at_bardo.clearlight.com
###############################################################################
###############################################################################
## (c) 2001-2008 Andr�s Garc�a Garc�a. fandom@telefonica.net
## Distributed under the terms of the GPL v2
###############################################################################
###############################################################################

package require tkwizard

tkwizard::tkwizard .wizard -title {Configure Getleft}

.wizard eval {
    variable wizData
    variable logo

    # default values
    if {[info exists wizData]} {
        unset wizData
    }

    set logo [image create photo \
            -file [file join "$dirGetleft(doc)" logo.gif]]

    bind .wizard <<WizCancel>> "
        tkwizard::.wizard::checkCancel
        break
    "
}

.wizard step {Language} -layout basic {
    global   getleftOptions supportedLang
    variable wizData

    set c [$this widget clientArea]

    foreach lang $supportedLang(langList) {
        set radio($lang) [ttk::radiobutton $c.radio$lang -text $lang   \
                -value $supportedLang($lang)                           \
                -variable getleftOptions(lang)                         \
                -command "tkwizard::.wizard::changeLanguage $supportedLang($lang)"]
    }

    if {![info exists getleftOptions(lang)]} {
        set getleftOptions(lang) en
    }
    changeLanguage $getleftOptions(lang)

    set i 0
    set j 0
    foreach lang $supportedLang(langList) {
        grid $radio($lang) -padx 5 -row $i -column $j -sticky w
        incr j
        if {$j==3} {
            set j 0
            incr i
        }
    }
    # In case we come back here from the proxy step, we don't want
    # to invoke the code to check the proxy values.
    bind .wizard <<WizNextStep>> ""
    return
}

if {$::wizFull==1} {
    .wizard step {installDir} -layout basic {
        variable wizData
        global logo env options confWizardLabels

        set c [$this widget clientArea]

        set forAll [ttk::radiobutton $c.forAll -text $confWizardLabels(howAll) \
                -variable installData(forAll) -value 1                         \
                -command "$c.path configure -state readonly"]
        set combo [ttk::combobox $c.path -values [list /usr /usr/local]        \
                -width 20 -state readonly -textvariable options(prefix)]
        set forMe  [ttk::radiobutton $c.forMe  -text $confWizardLabels(howMe)  \
                -variable installData(forAll) -value 0                         \
                -command "$c.path configure -state disabled"]
        $combo current 1
        set ::installData(forAll) 1

        grid $forAll -sticky w           -pady 5
        grid $combo             -padx 25 -pady {0 10}
        grid $forMe  -sticky w           -pady 5
        bind .wizard  <<WizNextStep>> {
            if {$installData(forAll)==1} {
                if {[ValidPath $options(prefix)]==1} {
                    Dialogos::Dialogo .wizard.error -title Error -type ok    \
                            -icon error -message $confWizardLabels(howBadPath)
                    break
                }
                if {[ValidPath $options(prefix)]==2} {
                    Dialogos::Dialogo .wizard.error -title Error -type ok    \
                            -icon error -message $confWizardLabels(howWritDir)
                    break
                }
            }
        }
        $this stepconfigure                                             \
            -title    $confWizardLabels(howTitle)                       \
            -subtitle $confWizardLabels(howSub)
        return
    }
}

.wizard step {Proxy} -layout basic {
    variable wizData
    global   confWizardLabels

    set c [$this widget clientArea]

    $this stepconfigure                          \
        -title    $confWizardLabels(proxTitle)   \
        -subtitle $confWizardLabels(proxSub)     \
        -pretext  {}                             \
        -posttext {}

    Herramientas::ConfProxyWindowCommon $c

    set useEvent <<WizFinish>>
    if {$::getleftState(os)=="unix"} {
        set useEvent <<WizNextStep>>
    }
    bind .wizard $useEvent "
        [namespace current]::proxyControl %W $useEvent
        break
    "
    return
}

if {($::getleftState(os)=="unix")} {
    .wizard step {Browser} -layout basic {
        variable wizData
        global   confWizardLabels

        set c [$this widget clientArea]

        $this stepconfigure                                 \
            -title    $confWizardLabels(browTitle)          \
            -subtitle $confWizardLabels(browSub)            \
            -pretext  {}                                    \
            -posttext {}

        Ayuda::ChooseLinuxBrowserCommon $c

        if {$::wizFull==1} {
            bind .wizard <<WizNextStep>> "
                [namespace current]::browserControl %W
            "
        } else {
            bind .wizard <<WizFinish>> "
                [namespace current]::browserControl %W
                Herramientas::SaveConfig
                break
            "
        }
        return
    }
}

if {$::wizFull==1} {
    .wizard step {ShortCuts} -layout basic {
        variable wizData
        global shorts icons confWizardLabels

        set c [$this widget clientArea]

        set lb1 [ttk::label $c.lb1 -text $confWizardLabels(warning1)]
        set lb2 [ttk::label $c.lb2 -text $confWizardLabels(warning2)]
        set lb3 [ttk::label $c.lb3 -text $confWizardLabels(warning3)]
        set menuIcon [ttk::checkbutton $c.mIcon -variable icons(mIcon) \
                -text $confWizardLabels(shortMenu)]
        set deskIcon [ttk::checkbutton $c.dIcon -variable icons(dIcon) \
                -text $confWizardLabels(shortDesk)]
        set icons(mIcon) 1
        set icons(dIcon) 1

        grid $lb1      -sticky w
        grid $lb2      -sticky w
        grid $lb3      -sticky w
        grid $menuIcon -sticky w -pady {20 7}
        grid $deskIcon -sticky w -pady {7 20}

        $this stepconfigure                             \
            -title    $confWizardLabels(shortTitle)     \
            -subtitle $confWizardLabels(shortSub)

        bind .wizard <<WizFinish>> {
#            .wizard configure -cursor watch
            if {$icons(mIcon)==1} {
                Herramientas::CreateShortcut menu
            }
            if {$icons(dIcon)==1} {
                Herramientas::CreateShortcut desktop
            }
            destroy .wizard
            break
        }
        return
    }
}

################################################################################
# proxyControl
#     Invokes the procedure that checks that the data entered for the proxy
#     is consistent.
#
# Parameter
#     W: The wizard window as returned by the event.
#     event: Whether we finish with this or not
################################################################################
proc tkwizard::.wizard::proxyControl {W useEvent} {

    set exitCode [::Herramientas::ConfProxyControl accept $W]
    if {$exitCode==0} {
        tkwizard::handleEvent $W $useEvent
        if {$useEvent=="<<WizFinish>>"} {
            destroy .wizard
        }
    }

    return
}

################################################################################
# browserControl
#     Checks whether all is well with the chose browser.
#
# Parameter
#     W: The wizard window as returned by the event.
################################################################################
proc tkwizard::.wizard::browserControl {W} {
    global wizFull
    set exitCode [::Ayuda::ChooseLinuxBrowserControl 1 $W]
    if {($exitCode==0) && ($wizFull==0)} {
        destroy .wizard
    }

    return
}

################################################################################
# changeLanguage
#     When the user click on one of the language options, this procedure takes
#     care of actually changing the texts.
#
# Parameter
#     lan: code of the language, 'es', 'en', ...
################################################################################
proc tkwizard::.wizard::changeLanguage {lan} {
    global confWizardLabels getleftOptions
    variable logo

    Herramientas::ReadLangFile $lan

    wm title .wizard $confWizardLabels(title)

    .wizard stepconfigure                                   \
        -title    $confWizardLabels(lanTitle)               \
        -subtitle $confWizardLabels(lanSub)                 \
        -icon     $logo                                     \
        -pretext  {}                                        \
        -posttext {}

    return
}

################################################################################
# checkCancel
#     When the user click on cancel or tries to close the window, we will first
#     check that he really means it.
################################################################################
proc tkwizard::.wizard::checkCancel {} {
    global confWizardLabels

    set what [Dialogos::Dialogo .wizard.question -type yesno -icon question \
            -message $confWizardLabels(sureMssg)                            \
            -title $confWizardLabels(sureTitle)]

    if {$what!="no"} {
        exit
    }

    focus .wizard
    return
}

.wizard show
tkwait window .wizard

